import 'package:ez_flutter_common/config/ez_config.dart';
import 'package:ez_flutter_common/config/ez_load_cfg.dart';
import 'package:ez_flutter_common/utils/ez_time_util.dart';
import 'package:ez_flutter_common/widgets/overlay/ez_overlay.dart';
import 'package:flutter/material.dart';


class EzDialog extends EzOverlay{

  EzDialog({
    this.builder,
    this.duration,
    this.bgColor: Colors.black12,
    this.hasBg: false,
    this.tapClose: false,
  }) : super(builder);

  WidgetBuilder builder;
  final int duration;
  final bool hasBg;
  final Color bgColor;
  final bool tapClose;

  show(BuildContext context,){
    this.showOverlay(context, builder: hasBg ? _showBgView : null);
    if(duration != null && duration > 0){
      EzSetTimeOut(duration, (){
        this.closeOverlay();
      });
    }
  }

  close(){
    this.closeOverlay();
  }

  Widget _showBgView(BuildContext context){
    return Container(
      color: bgColor,
      child: this.builder(context),
    );
  }
}
typedef EzDialog EzBaseDialogFun({String text,Widget child,int duration,double radius,bool hasBg});
typedef Widget EzBaseDialogCenterView({Widget child,String text,double radius});
class EzDialogCfg{


  static EzBaseDialogFun loading = _defLoading;
  static Function({String text,Widget child,int duration,double radius,bool hasBg}) success = _defSuccess;
  static EzBaseDialogFun fail = _defFail;
  static EzBaseDialogFun toast = _defToast;
  static EzBaseDialogCenterView centerView = _centerView;

  static int defTaostDuration = 1500;
  static int defSuccessDuration = 1500;
  static int defFialDuration = 1500;
  static double defTaostRadius = EzG.gW(5);

  static double defCenterRadius = EzG.gW(5);
  static Color defCenterBgColor = Colors.black12;
  static Color defViewColor = Color(0xff19191f);

  static EzDialog _defLoading({String text: "正在加载",Widget child,int duration,double radius,bool hasBg: true,Color bgColor}){
    return EzDialog(
        hasBg: hasBg,
        bgColor: bgColor ?? defCenterBgColor,
        builder: (BuildContext context){
          return centerView(
              radius: radius ?? defCenterRadius,
              child: EzLoadCfg.loadViewFun(),
              text: text
          );
        }
    );
  }

  static EzDialog _defSuccess({String text: "处理成功!",Widget child,int duration,double radius,bool hasBg: true,Color bgColor}){
    return EzDialog(
        hasBg: hasBg,
        duration: duration ?? defSuccessDuration,
        bgColor: bgColor ?? defCenterBgColor,
        builder: (BuildContext context){
          return centerView(
              radius: radius ?? defCenterRadius,
              child: Center(
                child: Transform.scale(scale: 0.9,child:  EzLoadCfg.rightViewFun(),),
              ),
              text: text
          );
        }
    );
  }

  static EzDialog _defFail({String text: "处理失败!请重试dasdkasldkaslkdkas大结局打卡时间肯德基萨克金短裤啊的",Widget child,int duration,double radius,bool hasBg: true,Color bgColor, Color viewColor}){
    return _defToast(
        hasBg: hasBg,
        child:  child ??  Row(
          children: [
            Expanded(child: Container(),),
            Container(
              padding: EdgeInsets.only(top: EzG.gW(12),bottom: EzG.gW(16),left: EzG.gW(26),right: EzG.gW(26)),
              decoration: BoxDecoration(
                  color: viewColor ?? defViewColor,
                  borderRadius: BorderRadius.all(Radius.circular(radius ?? defTaostRadius))
              ),
              child: Row(
                children: [
                  Container(
                    margin: EdgeInsets.only(right: EzG.gW(10)),
                    height: EzG.gW(36),
                    width: EzG.gW(36),
                    child: Icon(Icons.error_outline,size: EzG.gW(36),color: Colors.deepOrangeAccent,),
                  ),
                  Container(
                    child: Text("${text ?? "然而什么都没有"}", style: TextStyle(
                        color: Colors.white,fontSize: EzG.gSp(27),height: 1.6,letterSpacing: EzG.gW(1)
                    ),),
                  ),
                ],
              ),
            ),
            Expanded(child: Container(),),
          ],
        )
    );
  }

  static Widget _centerView({Widget child,String text,double radius}){
    return Center(
      child: Container(
        height: EzG.gW(180),
        width: EzG.gW(180),
        decoration: BoxDecoration(
            color: defViewColor,
            borderRadius: BorderRadius.all(Radius.circular(radius ?? defCenterRadius))
        ),
        child: Column(
          children: [
            Center(
              child: Container(
                margin: EdgeInsets.only(top: EzG.gW(25),),
                height: EzG.gW(75),
                width: EzG.gW(85),
                child: child,
              ),
            ),
            Container(
              margin: EdgeInsets.only(top: EzG.gW(18),),
              child: Text("${text}",style: TextStyle(
                  letterSpacing: EzG.gW(0.8),
                  color: Color(0xfcdedfe0),fontSize: EzG.gSp(25)
              ),),
            )
          ],
        ),
      ),
    );
  }

  static EzDialog _defToast({String text,Widget child,int duration,double radius,
    bool hasBg: false,Color viewColor}){
    return EzDialog(
        hasBg: hasBg,
        duration: duration ?? defTaostDuration,
        builder: (BuildContext context){
          return Container(
            width: MediaQuery.of(context).size.width,
            padding: EdgeInsets.all(EzG.gW(60)),
            child: Center(
              child: child ?? Container(
                padding: EdgeInsets.only(top: EzG.gW(12),bottom: EzG.gW(16),left: EzG.gW(26),right: EzG.gW(26)),
                child: Text("${text ?? "然而什么都没有"}", style: TextStyle(
                    color: Colors.white,fontSize: EzG.gSp(27),height: 1.6,letterSpacing: EzG.gW(1)
                ),),
                decoration: BoxDecoration(
                    color: viewColor ?? defViewColor,
                    borderRadius: BorderRadius.all(Radius.circular(radius ?? defTaostRadius))
                ),
              ),
            ),
          );
        }
    );
  }
}