import 'package:ez_flutter_common/config/ez_color.dart';
import 'package:ez_flutter_common/config/ez_config.dart';
import 'package:ez_flutter_common/config/ez_load_cfg.dart';
import 'package:ez_flutter_common/widgets/refresh/ez_def_rehresh_header.dart';
import 'package:ez_flutter_common/widgets/view/ez_state_view.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'ez_pagelist_controller.dart';

class EzPagelistView extends StatefulWidget{

  RefreshController refreshController;
  EzPagelistController plController;
  Widget Function(BuildContext context,List listData,EzPagelistController plController) builder;
  EzPagelistLoadDataFun loadDataFun;
  EzStateVCfg ezStateVCfg;
  EzListVCfg ezListVCfg;
  int pageSize;
  // 以下参数皆为 pull_to_refresh => SmartRefresher参数
  bool enablePullDown;
  bool enablePullUp;
  bool enableTwoLevel;
  Function onRefresh;
  Function onLoading;
  DragStartBehavior dragStartBehavior;
  bool primary;
  double cacheExtent;
  int semanticChildCount;
  bool reverse;
  ScrollPhysics physics;
  Axis scrollDirection;
  ScrollController scrollController;


  EzPagelistView({
    this.pageSize: 20,
    this.enablePullDown:true,
    this.enablePullUp:true,
    this.enableTwoLevel: false,
    this.onLoading,
    this.onRefresh,
    this.plController,
    this.builder,
    this.ezStateVCfg,
    this.ezListVCfg,
    this.dragStartBehavior,
    this.primary,
    this.cacheExtent,
    this.semanticChildCount,
    this.reverse,
    this.physics,
    this.scrollDirection,
    this.scrollController,
    this.loadDataFun}) {

    if(ezListVCfg == null){
      ezListVCfg = EzListVCfg.defCfg;
    }
    if(ezStateVCfg == null){
      ezStateVCfg = EzListVCfg.defCfg.ezStateVCfg;
    }
  }

  @override
  State<StatefulWidget> createState() {
    // TODO: implement createState
    return EzPagelistViewState();
  }
}

class EzPagelistViewState extends State<EzPagelistView>{
  RefreshController _refreshController;
  EzPagelistController _plController;

  get plController => widget.plController ?? _plController;
  get refreshController => widget.refreshController ?? _refreshController;


  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    if(plController == null){
      _plController = new EzPagelistController(widget.loadDataFun,pageSize: widget.pageSize)..init();
    }
    if(refreshController == null){
      _refreshController = new  RefreshController(initialRefresh: false);
    }
  }


  void _onRefresh() async{
    await plController.refresh();
    if(widget.onRefresh != null){
      await widget.onRefresh();
    }
    refreshController.refreshCompleted();
    refreshController.loadComplete();
  }

  void _onLoading() async{
    await plController.loadMore();
    if(widget.onLoading != null){
     await widget.onLoading();
    }
    if(plController.loadState.isNomore){
      refreshController.loadNoData();
    }else{
      refreshController.loadComplete();
    }
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return EzStateView(
        plController.loadState,
        reloadFun: plController.refresh,
        loadViewFun: widget.ezStateVCfg.loadViewFun,
        faildViewFun: widget.ezStateVCfg.faildViewFun,
        nodataViewFun: widget.ezStateVCfg.nodataViewFun,
        builder: (c){
          return ValueListenableBuilder(
              valueListenable: plController.valueVn,
              builder: (c,v,_child){
                return SmartRefresher(
                  child: widget.builder(context,plController.listData,plController),
                  enablePullDown: widget.enablePullDown,
                  enablePullUp: widget.enablePullUp,
                  enableTwoLevel: widget.enableTwoLevel,
                  header: widget.ezListVCfg.header ?? EzListVCfg.defCfg.header,
                  footer: widget.ezListVCfg.footer ?? EzListVCfg.defCfg.footer,
                  controller: refreshController,
                  onRefresh: _onRefresh,
                  onLoading: _onLoading,
                  dragStartBehavior: widget.dragStartBehavior,
                  primary: widget.primary,
                  cacheExtent: widget.cacheExtent,
                  semanticChildCount: widget.semanticChildCount,
                  reverse: widget.reverse,
                  physics: widget.physics,
                  scrollDirection: widget.scrollDirection,
                  scrollController: widget.scrollController,
                );
              });
        }
    );
  }
}

class EzListVCfg{
  static EzListVCfg defCfg = EzListVCfg(
      header: EzDefRehreshHeader(),
      footer: CustomFooter(
        builder: (BuildContext context,LoadStatus mode){
          Widget body ;
          if(mode==LoadStatus.loading){
            body =  SizedBox(
              width: EzG.gSp(70),
              height: EzG.gSp(70),
              child: EzLoadCfg.loadViewFun(),
            );
          }else{
            String _text = "上拉加载更多";
            if(mode==LoadStatus.idle){
              _text = LoadStatus_idle_text;
            }
            else if(mode == LoadStatus.failed){
              _text =  LoadStatus_failed_text;
            }
            else if(mode == LoadStatus.canLoading){
              _text = LoadStatus_canLoading_text;
            }
            else{
              _text = LoadStatus_nomore_text;
            } 
            body = Text(_text, style: TextStyle(
              color: EzG.getC(EzColor.T_C_3),
              fontSize: EzG.gSp(24)
            ),);
          }
          return Container(
            height: EzG.gW(100),
            child: Center(child:body),
          );
        },
      ),
      ezStateVCfg: EzStateVCfg.defCfg
  );

  static setDefCfg(EzListVCfg cfg){defCfg = cfg;}

  static String LoadStatus_idle_text = "上拉加载更多";
  static String LoadStatus_failed_text = "加载失败！请重试";
  static String LoadStatus_canLoading_text = "松开手指进行加载";
  static String LoadStatus_nomore_text = "暂无更多内容";

  static String refreshStatus_complete_text = "刷新完成";
  int pageSize;
  Widget header;
  Widget footer;
  EzStateVCfg ezStateVCfg;
  EzListVCfg({this.footer,this.header,this.ezStateVCfg});
}